--- old/usr/src/uts/intel/ia32/os/sendsig.c	2013-01-23 13:19:08.913729112 +0000
+++ new/usr/src/uts/intel/ia32/os/sendsig.c	2013-01-23 13:19:08.922717765 +0000
@@ -133,13 +133,15 @@
 	caddr_t fp;
 	volatile struct regs *rp;
 	volatile greg_t upc;
-	volatile proc_t *p = ttoproc(curthread);
+	proc_t *volatile p = ttoproc(curthread);
 	struct as *as = p->p_as;
 	klwp_t *lwp = ttolwp(curthread);
 	ucontext_t *volatile tuc = NULL;
 	ucontext_t *uc;
 	siginfo_t *sip_addr;
 	volatile int watched;
+	char *volatile xregs = NULL;
+	volatile size_t xregs_size = 0;
 
 	/*
 	 * This routine is utterly dependent upon STACK_ALIGN being
@@ -175,6 +177,14 @@
 	minstacksz = sizeof (struct sigframe) + SA(sizeof (*uc));
 	if (sip != NULL)
 		minstacksz += SA(sizeof (siginfo_t));
+
+	/*
+	 * Extra registers, if supported by this platform, may be of arbitrary
+	 * length. Size them now so we know how big the signal frame has to be.
+	 */
+	xregs_size = xregs_getsize(p);
+	minstacksz += SA(xregs_size);
+
 	ASSERT((minstacksz & (STACK_ENTRY_ALIGN - 1ul)) == 0);
 
 	/*
@@ -289,6 +299,20 @@
 	uc = (ucontext_t *)(sp + sizeof (struct sigframe));
 	tuc = kmem_alloc(sizeof (*tuc), KM_SLEEP);
 	savecontext(tuc, &lwp->lwp_sigoldmask);
+
+	/*
+	 * Save extra register state if it exists.
+	 */
+	if (xregs_size != 0) {
+		xregs_setptr(lwp, tuc, sp);
+		xregs = kmem_alloc(xregs_size, KM_SLEEP);
+		xregs_get(lwp, xregs);
+		copyout_noerr(xregs, sp, xregs_size);
+		kmem_free(xregs, xregs_size);
+		xregs = NULL;
+		sp += SA(xregs_size);
+	}
+
 	copyout_noerr(tuc, uc, sizeof (*tuc));
 	kmem_free(tuc, sizeof (*tuc));
 	tuc = NULL;
@@ -354,6 +378,8 @@
 		watch_enable_addr((caddr_t)sp, minstacksz, S_WRITE);
 	if (tuc)
 		kmem_free(tuc, sizeof (*tuc));
+	if (xregs)
+		kmem_free(xregs, xregs_size);
 #ifdef DEBUG
 	printf("sendsig: bad signal stack cmd=%s, pid=%d, sig=%d\n",
 	    PTOU(p)->u_comm, p->p_pid, sig);
@@ -393,12 +419,14 @@
 	caddr_t fp;
 	volatile struct regs *rp;
 	volatile greg_t upc;
-	volatile proc_t *p = ttoproc(curthread);
+	proc_t *volatile p = ttoproc(curthread);
 	klwp_t *lwp = ttolwp(curthread);
 	ucontext32_t *volatile tuc = NULL;
 	ucontext32_t *uc;
 	siginfo32_t *sip_addr;
 	volatile int watched;
+	char *volatile xregs = NULL;
+	volatile size_t xregs_size = 0;
 
 	rp = lwptoregs(lwp);
 	upc = rp->r_pc;
@@ -406,6 +434,14 @@
 	minstacksz = SA32(sizeof (struct sigframe32)) + SA32(sizeof (*uc));
 	if (sip != NULL)
 		minstacksz += SA32(sizeof (siginfo32_t));
+
+	/*
+	 * Extra registers, if supported by this platform, may be of arbitrary
+	 * length. Size them now so we know how big the signal frame has to be.
+	 */
+	xregs_size = xregs_getsize(p);
+	minstacksz += SA32(xregs_size);
+
 	ASSERT((minstacksz & (STACK_ALIGN32 - 1)) == 0);
 
 	/*
@@ -507,6 +543,20 @@
 	uc = (ucontext32_t *)fp;
 	tuc = kmem_alloc(sizeof (*tuc), KM_SLEEP);
 	savecontext32(tuc, &lwp->lwp_sigoldmask);
+
+	/*
+	 * Save extra register state if it exists.
+	 */
+	if (xregs_size != 0) {
+		xregs_setptr32(lwp, tuc, (caddr32_t)(uintptr_t)sp);
+		xregs = kmem_alloc(xregs_size, KM_SLEEP);
+		xregs_get(lwp, xregs);
+		copyout_noerr(xregs, sp, xregs_size);
+		kmem_free(xregs, xregs_size);
+		xregs = NULL;
+		sp += SA32(xregs_size);
+	}
+
 	copyout_noerr(tuc, uc, sizeof (*tuc));
 	kmem_free(tuc, sizeof (*tuc));
 	tuc = NULL;
@@ -572,6 +622,8 @@
 		watch_enable_addr((caddr_t)sp, minstacksz, S_WRITE);
 	if (tuc)
 		kmem_free(tuc, sizeof (*tuc));
+	if (xregs_size)
+		kmem_free(xregs, xregs_size);
 #ifdef DEBUG
 	printf("sendsig32: bad signal stack cmd=%s pid=%d, sig=%d\n",
 	    PTOU(p)->u_comm, p->p_pid, sig);
